package il.ac.tau.cs.sw1.sudoku;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class GUI {
	int[][] matrix = new int[9][9];
	private Shell shell = null;
	
	private Text[][] cells; 	// individual cells(9x9)
	private Group[][] blocks; 	// 3x3 blocks (for border)
	private Composite table; 	// whole table
	
	public void init(){
		Display display = new Display ();
		shell = new Shell (display, SWT.DIALOG_TRIM);
		shell.setText("Soduku");		
		Layout layout = new GridLayout(1,false);		
 		shell.setLayout(layout); 	 			
		
 		Label explanation = new Label(shell, SWT.CENTER);
 		explanation.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_CENTER, GridData.VERTICAL_ALIGN_CENTER, false, false));
 		explanation.setText("Fill numbers in selected cells and press 'solve'");
 		
 		buildTable();
 		addButtons();

 		
		shell.pack ();
		shell.open ();
		while (!shell.isDisposed ()) {
			if (!display.readAndDispatch ()) display.sleep ();
		}
		display.dispose ();
	}
	

	private void buildTable() {		
		
		table = new Composite(shell, SWT.BORDER);
		table.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_CENTER, GridData.VERTICAL_ALIGN_CENTER, true, true));
		table.setLayout(new GridLayout(3, true));
		
		blocks = new Group[3][3];
		
		for (int i=0;i<3;i++) {
			for (int j=0;j<3;j++) {
				blocks[i][j] = new Group(table, SWT.SHADOW_NONE);
				blocks[i][j].setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_CENTER, GridData.VERTICAL_ALIGN_CENTER, false, false));
				blocks[i][j].setLayout(new GridLayout(3, true));
			}
		}		
		
		cells = new Text[9][9];
		
		for (int i=0;i<9;i++) {
			for (int j=0;j<9;j++) {
				cells[i][j] = new Text(blocks[i/3][j/3],SWT.BORDER);
				cells[i][j].setSize(5,cells[i][j].getSize().y);
				cells[i][j].setLayoutData(new GridData(12,10));
				//cells[i][j].setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_CENTER, GridData.VERTICAL_ALIGN_CENTER, false, false));
				cells[i][j].addVerifyListener(new VerifyListener() {
					public void verifyText(VerifyEvent e) {
						if (e.start > 0 || !is1to9Digit(e.text)) {
							e.text = "";
						}
					}
				});
			}
		}
	}
	
	private int[][] getMatrix() {
		int[][] m = new int[9][9];
		
		for (int i=0; i <9; i++){
			for (int j=0; j < 9; j++){
				String s= /*table.getItem(i).getText(j)*/cells[i][j].getText();
				if (s.equals("")){
					m[i][j]=-1;
				}else{
					m[i][j] = Integer.parseInt(/*table.getItem(i).getText(j)*/cells[i][j].getText());
				}
			}
		}
		
		return m;
	}
	
	private void fillMatrix(int[][] m) {
		for (int i=0; i <9; i++){
			for (int j=0; j < 9; j++){											
				cells[i][j].setText(Integer.toString(m[i][j]));
			}
		}
	}
	
	private void clearMatrix() {
		for (int i=0; i <9; i++){
			for (int j=0; j < 9; j++){
				cells[i][j].setText("");
			}
		}
	}
	
	private void addButtons() {	
		Composite buttons = new Composite(shell,SWT.SHADOW_NONE);
		FillLayout fillLayout = new FillLayout(SWT.HORIZONTAL);
		fillLayout.spacing = 3;
		buttons.setLayout(fillLayout );
		buttons.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING, GridData.VERTICAL_ALIGN_CENTER, false, false));
		Button button = new Button (buttons, SWT.PUSH);		
		button.setText("Solve!!!");
		//button.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING, GridData.VERTICAL_ALIGN_CENTER, false, false));
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				matrix = getMatrix();
				
				if (!Solution.calcSolution(matrix)){
					MessageBox msgBox = new MessageBox(shell);
					msgBox.setMessage("No solution");
					msgBox.open();
					
					clearMatrix();
				}else{
					fillMatrix(matrix);					
				}
				
			}
		});
		
		
		Button button2 = new Button (buttons, SWT.PUSH);		
		button2.setText("Clear");
		//button2.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING, GridData.VERTICAL_ALIGN_CENTER, false, false));
		button2.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				clearMatrix();
			}
		});	
	}
	private static boolean is1to9Digit(String s){
		if (s.length()!=1) return false;
		char c = s.charAt(0);
		return ((c >= '1') && (c <= '9'));
	}

	public static void main(String[] args) {			
		new GUI().init();
	}

}
